@@ -4,6 +4,7 @@ import importlib
 import inspect
 import os
 import warnings
+from collections import defaultdict
 from contextlib import contextmanager
 from contextlib import suppress
 from pathlib import Path
@@ -12,6 +13,7 @@ from dynaconf import default_settings
 from dynaconf.loaders import default_loader
 from dynaconf.loaders import enable_external_loaders
 from dynaconf.loaders import env_loader
+from dynaconf.loaders import execute_hooks
 from dynaconf.loaders import py_loader
 from dynaconf.loaders import settings_loader
 from dynaconf.loaders import yaml_loader
@@ -36,60 +38,17 @@ from dynaconf.vendor.box.box_list import BoxList
 
 
 class LazySettings(LazyObject):
-    """When you do::
+    """Loads settings lazily from multiple sources::
 
-        >>> from dynaconf import Dynaconf
-        >>> settings = Dynaconf(*options)
+        settings = Dynaconf(
+            settings_files=["settings.toml"],  # path/glob
+            environments=True,                 # activate layered environments
+            envvar_prefix="MYAPP",             # `export MYAPP_FOO=bar`
+            env_switcher="MYAPP_MODE",         # `export MYAPP_MODE=production`
+            load_dotenv=True,                  # read a .env file
+        )
 
-    a LazySettings is initialized with only default_settings and all the
-    parameters passed to *options.
-
-    Then when you first access a value, this will be set up and loaders will
-    be executed looking for defined config files or the file defined in
-    SETTINGS_FILE_FOR_DYNACONF environment variable::
-
-        $ export SETTINGS_FILE_FOR_DYNACONF=path
-
-        >>> settings.FOO
-
-    Or when you call::
-
-        >>> settings.configure(settings_module='/tmp/settings.py')
-
-    You can define in your settings module a list of loaders to get values
-    from different stores. By default it will try environment variables
-    starting with ENVVAR_PREFIX_FOR_DYNACONF (by defaulf `DYNACONF_`)
-
-    You can also customize specific parameters.
-
-    Exmaple: `proj/config.py`::
-
-        >>> from dynaconf import Dynaconf
-        >>> settings = Dynaconf(
-        ...    env='production',
-        ...    loaders=[
-        ...        'dynaconf.loaders.env_loader',
-        ...        'dynaconf.loaders.redis_loader'
-        ...    ]
-        ... )
-
-    save common values in a settings file::
-
-        $ echo "SERVER_IP = '10.10.10.10'" > proj/settings.py
-
-    or use `.toml|.yaml|.ini|.json`
-
-    save sensitive values in .secrets.{py|toml|yaml|ini|json}
-    or export as DYNACONF global environment variable::
-
-        $ export DYNACONF_SERVER_PASSWD='super_secret'
-
-        >>> # from proj.config import settings
-        >>> print settings.SERVER_IP
-        >>> print settings.SERVER_PASSWD
-
-    and now it reads all variables starting with `DYNACONF_` from envvars
-    and all values in a hash called DYNACONF_PROJ in redis
+    More options available on https://www.dynaconf.com/configuration/
     """
 
     def __init__(self, wrapped=None, **kwargs):
@@ -185,6 +144,15 @@ class LazySettings(LazyObject):
         """
         return self.get(*args, **kwargs)
 
+    @property
+    def _should_load_dotenv(self):
+        """Chicken and egg problem, we must manually check envvar
+        before deciding if we are loading envvars :)"""
+        _environ_load_dotenv = parse_conf_data(
+            os.environ.get("LOAD_DOTENV_FOR_DYNACONF"), tomlfy=True
+        )
+        return self._kwargs.get("load_dotenv", _environ_load_dotenv)
+
     def _setup(self):
         """Initial setup, run once."""
 
@@ -196,7 +164,7 @@ class LazySettings(LazyObject):
                 DeprecationWarning,
             )
 
-        default_settings.reload(self._kwargs.get("load_dotenv"))
+        default_settings.reload(self._should_load_dotenv)
         environment_variable = self._kwargs.get(
             "ENVVAR_FOR_DYNACONF", default_settings.ENVVAR_FOR_DYNACONF
         )
@@ -213,7 +181,7 @@ class LazySettings(LazyObject):
         :param settings_module: defines the setttings file
         :param kwargs:  override default settings
         """
-        default_settings.reload(self._kwargs.get("load_dotenv"))
+        default_settings.reload(self._should_load_dotenv)
         environment_var = self._kwargs.get(
             "ENVVAR_FOR_DYNACONF", default_settings.ENVVAR_FOR_DYNACONF
         )
@@ -244,6 +212,8 @@ class Settings:
         """
         self._fresh = False
         self._loaded_envs = []
+        self._loaded_hooks = defaultdict(dict)
+        self._loaded_py_modules = []
         self._loaded_files = []
         self._deleted = set()
         self._store = DynaBox(box_settings=self)
@@ -253,7 +223,13 @@ class Settings:
         self._defaults = DynaBox(box_settings=self)
         self.environ = os.environ
         self.SETTINGS_MODULE = None
+        self.filter_strategy = kwargs.get("filter_strategy", None)
         self._not_installed_warnings = []
+        self._validate_only = kwargs.pop("validate_only", None)
+        self._validate_exclude = kwargs.pop("validate_exclude", None)
+        self._validate_only_current_env = kwargs.pop(
+            "validate_only_current_env", False
+        )
 
         self.validators = ValidatorList(
             self, validators=kwargs.pop("validators", None)
@@ -266,9 +242,20 @@ class Settings:
             self.set(key, value)
         # execute loaders only after setting defaults got from kwargs
         self._defaults = kwargs
-        self.execute_loaders()
 
-        self.validators.validate()
+        # The following flags are used for when copying of settings is done
+        skip_loaders = kwargs.get("dynaconf_skip_loaders", False)
+        skip_validators = kwargs.get("dynaconf_skip_validators", False)
+
+        if not skip_loaders:
+            self.execute_loaders()
+
+        if not skip_validators:
+            self.validators.validate(
+                only=self._validate_only,
+                exclude=self._validate_exclude,
+                only_current_env=self._validate_only_current_env,
+            )
 
     def __call__(self, *args, **kwargs):
         """Allow direct call of `settings('val')`
@@ -350,18 +337,37 @@ class Settings:
         """Redirects to store object"""
         return self.store.values()
 
-    def setdefault(self, item, default):
-        """Returns value if exists or set it as the given default"""
+    def setdefault(self, item, default, apply_default_on_none=False):
+        """Returns value if exists or set it as the given default
+
+        apply_default_on_none: if True, default is set when value is None
+        """
         value = self.get(item, empty)
-        if value is empty and default is not empty:
+
+        # Yaml loader reads empty values as None, whould we apply defaults?
+        global_apply_default = (
+            self.get("APPLY_DEFAULT_ON_NONE_FOR_DYNACONF") is not None
+        )
+        apply_default = default is not empty and (
+            value is empty
+            or (
+                value is None
+                and (
+                    apply_default_on_none is True
+                    or global_apply_default is True
+                )
+            )
+        )
+
+        if apply_default:
             self.set(
                 item,
                 default,
                 loader_identifier="setdefault",
                 tomlfy=True,
-                dotted_lookup=True,
             )
             return default
+
         return value
 
     def as_dict(self, env=None, internal=False):
@@ -413,7 +419,7 @@ class Settings:
         default=None,
         cast=None,
         fresh=False,
-        dotted_lookup=True,
+        dotted_lookup=empty,
         parent=None,
     ):
         """
@@ -435,6 +441,9 @@ class Settings:
             # turn FOO__bar__ZAZ in `FOO.bar.ZAZ`
             key = key.replace(nested_sep, ".")
 
+        if dotted_lookup is empty:
+            dotted_lookup = self._store.get("DOTTED_LOOKUP_FOR_DYNACONF")
+
         if "." in key and dotted_lookup:
             return self._dotted_get(
                 dotted_key=key,
@@ -536,14 +545,14 @@ class Settings:
             self._loaded_envs = []
         return self._loaded_envs
 
-    # compat
-    loaded_namespaces = loaded_envs
-
     @loaded_envs.setter
     def loaded_envs(self, value):
         """Setter for env list"""
         self._loaded_envs = value
 
+    # compat
+    loaded_namespaces = loaded_envs
+
     @property
     def loaded_by_loaders(self):
         """Gets the internal mapping of LOADER -> values"""
@@ -587,6 +596,10 @@ class Settings:
             if key not in RENAMED_VARS
         }
 
+        if self.filter_strategy:
+            # Retain the filtering strategy when switching environments
+            new_data["filter_strategy"] = self.filter_strategy
+
         # This is here for backwards compatibility
         # To be removed on 4.x.x
         default_settings_paths = self.get("default_settings_paths")
@@ -676,7 +689,7 @@ class Settings:
         """Return the current active env"""
 
         if self.ENVIRONMENTS_FOR_DYNACONF is False:
-            return "main"
+            return self.MAIN_ENV_FOR_DYNACONF.lower()
 
         if self.FORCE_ENV_FOR_DYNACONF is not None:
             return self.FORCE_ENV_FOR_DYNACONF
@@ -823,7 +836,7 @@ class Settings:
         value,
         loader_identifier=None,
         tomlfy=False,
-        dotted_lookup=True,
+        dotted_lookup=empty,
         is_secret="DeprecatedArgument",  # noqa
         merge=False,
     ):
@@ -835,6 +848,9 @@ class Settings:
         :param tomlfy: Bool define if value is parsed by toml (defaults False)
         :param merge: Bool define if existing nested data will be merged.
         """
+        if dotted_lookup is empty:
+            dotted_lookup = self.get("DOTTED_LOOKUP_FOR_DYNACONF")
+
         nested_sep = self.get("NESTED_SEPARATOR_FOR_DYNACONF")
         if nested_sep and nested_sep in key:
             # turn FOO__bar__ZAZ in `FOO.bar.ZAZ`
@@ -856,9 +872,15 @@ class Settings:
 
         if getattr(value, "_dynaconf_reset", False):  # pragma: no cover
             # just in case someone use a `@reset` in a first level var.
-            # NOTE: @reset/Reset is deprecated in v3.0.0
             value = value.unwrap()
 
+        if getattr(value, "_dynaconf_merge_unique", False):
+            # just in case someone use a `@merge_unique` in a first level var
+            if existing:
+                value = object_merge(existing, value.unwrap(), unique=True)
+            else:
+                value = value.unwrap()
+
         if getattr(value, "_dynaconf_merge", False):
             # just in case someone use a `@merge` in a first level var
             if existing:
@@ -872,6 +894,7 @@ class Settings:
                 value = object_merge(existing, value)
             else:
                 # `dynaconf_merge` may be used within the key structure
+                # Or merge_enabled is set to True
                 value = self._merge_before_set(existing, value)
 
         if isinstance(value, dict):
@@ -898,6 +921,7 @@ class Settings:
         tomlfy=False,
         merge=False,
         is_secret="DeprecatedArgument",  # noqa
+        dotted_lookup=empty,
         **kwargs,
     ):
         """
@@ -928,6 +952,7 @@ class Settings:
                 loader_identifier=loader_identifier,
                 tomlfy=tomlfy,
                 merge=merge,
+                dotted_lookup=dotted_lookup,
             )
 
     def _merge_before_set(self, existing, value):
@@ -1003,10 +1028,11 @@ class Settings:
 
             loaders = self.loaders
 
-        for loader in loaders:
-            loader.load(self, env, silent=silent, key=key)
+        for core_loader in loaders:
+            core_loader.load(self, env, silent=silent, key=key)
 
         self.load_includes(env, silent=silent, key=key)
+        execute_hooks("post", self, env, silent=silent, key=key)
 
     def pre_load(self, env, silent, key):
         """Do we have any file to pre-load before main settings file?"""
@@ -1170,7 +1196,15 @@ class Settings:
 
     def dynaconf_clone(self):
         """Clone the current settings object."""
-        return copy.deepcopy(self)
+        try:
+            return copy.deepcopy(self)
+        except TypeError:
+            # can't deepcopy settings object bacause of module object
+            # being set as value in the settings dict
+            new_data = self.to_dict(internal=True)
+            new_data["dynaconf_skip_loaders"] = True
+            new_data["dynaconf_skip_validators"] = True
+            return Settings(**new_data)
 
     @property
     def dynaconf(self):
@@ -1230,13 +1264,20 @@ RESERVED_ATTRS = (
         "_kwargs",
         "_loaded_by_loaders",
         "_loaded_envs",
+        "_loaded_hooks",
+        "_loaded_py_modules",
         "_loaded_files",
         "_loaders",
         "_not_installed_warnings",
         "_store",
         "_warn_dynaconf_global_settings",
+        "_should_load_dotenv",
         "environ",
         "SETTINGS_MODULE",
+        "filter_strategy",
         "validators",
+        "_validate_only",
+        "_validate_exclude",
+        "_validate_only_current_env",
     ]
 )
