@@ -18,11 +18,13 @@ from dynaconf.utils import upperfy
 from dynaconf.utils.files import read_file
 from dynaconf.utils.functional import empty
 from dynaconf.utils.parse_conf import parse_conf_data
+from dynaconf.utils.parse_conf import unparse_conf_data
 from dynaconf.validator import ValidationError
 from dynaconf.validator import Validator
 from dynaconf.vendor import click
 from dynaconf.vendor import toml
 
+os.environ["PYTHONIOENCODING"] = "utf-8"
 
 CWD = Path.cwd()
 EXTS = ["ini", "toml", "yaml", "json", "py", "env"]
@@ -39,6 +41,10 @@ def set_settings(ctx, instance=None):
     settings = None
 
     if instance is not None:
+        if ctx.invoked_subcommand in ["init"]:
+            raise click.UsageError(
+                "-i/--instance option is not allowed for `init` command"
+            )
         sys.path.insert(0, ".")
         settings = import_settings(instance)
     elif "FLASK_APP" in os.environ:  # pragma: no cover
@@ -59,13 +65,8 @@ def set_settings(ctx, instance=None):
             from django.conf import settings  # noqa
 
             settings.DYNACONF.configure()
-        except (ImportError, AttributeError):
-            # Backwards compatible with old django extension (pre 2.0.0)
-            import dynaconf.contrib.django_dynaconf  # noqa
-            from django.conf import settings as django_settings  # noqa
-
-            django_settings.configure()
-            settings = django_settings
+        except AttributeError:
+            settings = LazySettings()
 
         if settings is not None:
             click.echo(
@@ -211,7 +212,10 @@ def main(ctx, instance):
     "--path", "-p", default=CWD, help="defaults to current directory"
 )
 @click.option(
-    "--env", "-e", default=None, help="Sets the working env in `.env` file"
+    "--env",
+    "-e",
+    default=None,
+    help="deprecated command (kept for compatibility but unused)",
 )
 @click.option(
     "--vars",
@@ -221,7 +225,7 @@ def main(ctx, instance):
     default=None,
     help=(
         "extra values to write to settings file "
-        "file e.g: `dynaconf init -v NAME=foo -v X=2"
+        "e.g: `dynaconf init -v NAME=foo -v X=2`"
     ),
 )
 @click.option(
@@ -256,6 +260,15 @@ def init(ctx, fileformat, path, env, _vars, _secrets, wg, y, django):
     click.echo("-" * 42)
     path = Path(path)
 
+    if env is not None:
+        click.secho(
+            "⚠️ The --env/-e option is deprecated (kept for\n"
+            "   compatibility but unused)\n",
+            fg="red",
+            bold=True,
+            # stderr=True,
+        )
+
     if settings.get("create_new_settings") is True:
         filename = Path("config.py")
         if not filename.exists():
@@ -383,11 +396,50 @@ def init(ctx, fileformat, path, env, _vars, _secrets, wg, y, django):
             click.echo("🎠  Now your Django settings are managed by Dynaconf")
         else:
             click.echo("❌  Django settings file not written.")
+    else:
+        click.echo(
+            "🎉 Dynaconf is configured! read more on https://dynaconf.com\n"
+            "   Use `dynaconf -i config.settings list` to see your settings\n"
+        )
 
-    click.echo(
-        "🎉 Dynaconf is configured! read more on https://dynaconf.com\n"
-        "   Use `dynaconf -i config.settings list` to see your settings\n"
-    )
+
+@main.command(name="get")
+@click.argument("key", required=True)
+@click.option(
+    "--default",
+    "-d",
+    default=empty,
+    help="Default value if settings doesn't exist",
+)
+@click.option(
+    "--env", "-e", default=None, help="Filters the env to get the values"
+)
+@click.option(
+    "--unparse",
+    "-u",
+    default=False,
+    help="Unparse data by adding markers such as @none, @int etc..",
+    is_flag=True,
+)
+def get(key, default, env, unparse):
+    """Returns the raw value for a settings key"""
+    if env:
+        env = env.strip()
+    if key:
+        key = key.strip()
+
+    if env:
+        settings.setenv(env)
+
+    if default is not empty:
+        result = settings.get(key, default)
+    else:
+        result = settings[key]  # let the keyerror raises
+
+    if unparse:
+        result = unparse_conf_data(result)
+
+    click.echo(result, nl=False)
 
 
 @main.command(name="list")
