@@ -10,14 +10,41 @@ ALL_EXTENSIONS = (
 
 EXTERNAL_LOADERS = {
     "ENV": "dynaconf.loaders.env_loader",
-    "REDIS": "dynaconf.loaders.redis_loader",
     "VAULT": "dynaconf.loaders.vault_loader",
+    "REDIS": "dynaconf.loaders.redis_loader",
 }
 
 DJANGO_PATCH = """
 # HERE STARTS DYNACONF EXTENSION LOAD (Keep at the very bottom of settings.py)
-# Read more at https://dynaconf.readthedocs.io/en/latest/guides/django.html
+# Read more at https://www.dynaconf.com/django/
 import dynaconf  # noqa
 settings = dynaconf.DjangoDynaconf(__name__)  # noqa
 # HERE ENDS DYNACONF EXTENSION LOAD (No more code below this line)
  """
+
+INSTANCE_TEMPLATE = """
+from dynaconf import Dynaconf
+
+settings = Dynaconf(
+    envvar_prefix="DYNACONF",
+    settings_files={settings_files},
+)
+
+# `envvar_prefix` = export envvars with `export DYNACONF_FOO=bar`.
+# `settings_files` = Load these files in the order.
+"""
+
+EXTS = (
+    "py",
+    "toml",
+    "tml",
+    "yaml",
+    "yml",
+    "ini",
+    "conf",
+    "properties",
+    "json",
+)
+DEFAULT_SETTINGS_FILES = [f"settings.{ext}" for ext in EXTS] + [
+    f".secrets.{ext}" for ext in EXTS
+]
