@@ -1,4 +1,6 @@
 import warnings
+from collections import ChainMap
+from contextlib import suppress
 
 try:
     from flask.config import Config
@@ -10,7 +12,7 @@ except ImportError:  # pragma: no cover
 
 
 import dynaconf
-from importlib import import_module
+import pkg_resources
 
 
 class FlaskDynaconf:
@@ -88,8 +90,7 @@ class FlaskDynaconf:
                 "To use this extension Flask must be installed "
                 "install it with: pip install flask"
             )
-        self.kwargs = kwargs
-
+        self.kwargs = {k.upper(): v for k, v in kwargs.items()}
         kwargs.setdefault("ENVVAR_PREFIX", "FLASK")
         env_prefix = f"{kwargs['ENVVAR_PREFIX']}_ENV"  # FLASK_ENV
         kwargs.setdefault("ENV_SWITCHER", env_prefix)
@@ -136,26 +137,27 @@ class FlaskDynaconf:
 
 class DynaconfConfig(Config):
     """
-    Settings load order in Dynaconf:
-
-    - Load all defaults and Flask defaults
-    - Load all passed variables when applying FlaskDynaconf
-    - Update with data in settings files
-    - Update with data in environmente vars `ENV_FOR_DYNACONF_`
+    Replacement for flask.config_class that responds as a Dynaconf instance.
     """
 
     def __init__(self, _settings, _app, *args, **kwargs):
         """perform the initial load"""
         super(DynaconfConfig, self).__init__(*args, **kwargs)
+
+        # Bring Dynaconf instance value to Flask Config
         Config.update(self, _settings.store)
+
         self._settings = _settings
         self._app = _app
 
+    def __contains__(self, item):
+        return hasattr(self, item)
+
     def __getitem__(self, key):
-        """
-        Flask templates always expects a None when key is not found in config
-        """
-        return self.get(key)
+        try:
+            return self._settings[key]
+        except KeyError:
+            return Config.__getitem__(self, key)
 
     def __setitem__(self, key, value):
         """
@@ -163,15 +165,38 @@ class DynaconfConfig(Config):
         """
         return self._settings.__setitem__(key, value)
 
+    def _chain_map(self):
+        return ChainMap(self._settings, dict(dict.items(self)))
+
+    def keys(self):
+        return self._chain_map().keys()
+
+    def values(self):
+        return self._chain_map().values()
+
+    def items(self):
+        return self._chain_map().items()
+
+    def setdefault(self, key, value=None):
+        return self._chain_map().setdefault(key, value)
+
+    def __iter__(self):
+        return self._chain_map().__iter__()
+
     def __getattr__(self, name):
         """
-        First try to get value from dynaconf then from Flask
+        First try to get value from dynaconf then from Flask Config
         """
-        try:
+        with suppress(AttributeError):
             return getattr(self._settings, name)
-        except AttributeError:
+
+        with suppress(KeyError):
             return self[name]
 
+        raise AttributeError(
+            f"'{self.__class__.__name__}' object has no attribute '{name}'"
+        )
+
     def __call__(self, name, *args, **kwargs):
         return self.get(name, *args, **kwargs)
 
@@ -192,10 +217,12 @@ class DynaconfConfig(Config):
             )
             return
 
-        for extension in app.config[key]:
-            # Split data in form `extension.path:factory_function`
-            module_name, factory = extension.split(":")
-            # Dynamically import extension module.
-            ext = import_module(module_name)
-            # Invoke factory passing app.
-            getattr(ext, factory)(app)
+        for object_reference in app.config[key]:
+            # add a placeholder `name` to create a valid entry point
+            entry_point_spec = f"__name = {object_reference}"
+            # parse the entry point specification
+            entry_point = pkg_resources.EntryPoint.parse(entry_point_spec)
+            # dynamically resolve the entry point
+            initializer = entry_point.resolve()
+            # Invoke extension initializer
+            initializer(app)
