@@ -1,3 +1,4 @@
+import importlib
 import os
 
 from dynaconf import constants as ct
@@ -57,6 +58,80 @@ def default_loader(obj, defaults=None):
             obj.set(key, env_value, tomlfy=True)
 
 
+def _run_hook_module(hook, hook_module, obj, key=None):
+    """Run the hook function from the settings obj.
+
+    given a hook name, a hook_module and a settings object
+    load the function and execute if found.
+    """
+    if hook in obj._loaded_hooks.get(hook_module.__file__, {}):
+        # already loaded
+        return
+
+    if hook_module and getattr(hook_module, "_error", False):
+        if not isinstance(hook_module._error, FileNotFoundError):
+            raise hook_module._error
+
+    hook_func = getattr(hook_module, hook, None)
+    if hook_func:
+        hook_dict = hook_func(obj.dynaconf.clone())
+        if hook_dict:
+            merge = hook_dict.pop(
+                "dynaconf_merge", hook_dict.pop("DYNACONF_MERGE", False)
+            )
+            if key and key in hook_dict:
+                obj.set(key, hook_dict[key], tomlfy=False, merge=merge)
+            elif not key:
+                obj.update(hook_dict, tomlfy=False, merge=merge)
+        obj._loaded_hooks[hook_module.__file__][hook] = hook_dict
+
+
+def execute_hooks(
+    hook, obj, env=None, silent=True, key=None, modules=None, files=None
+):
+    """Execute dynaconf_hooks from module or filepath."""
+    if hook not in ["post"]:
+        raise ValueError(f"hook {hook} not supported yet.")
+
+    # try to load hooks using python module __name__
+    modules = modules or obj._loaded_py_modules
+    for loaded_module in modules:
+        hook_module_name = ".".join(
+            loaded_module.split(".")[:-1] + ["dynaconf_hooks"]
+        )
+        try:
+            hook_module = importlib.import_module(hook_module_name)
+        except (ImportError, TypeError):
+            # There was no hook on the same path as a python module
+            continue
+        else:
+            _run_hook_module(
+                hook=hook,
+                hook_module=hook_module,
+                obj=obj,
+                key=key,
+            )
+
+    # Try to load from python filename path
+    files = files or obj._loaded_files
+    for loaded_file in files:
+        hook_file = os.path.join(
+            os.path.dirname(loaded_file), "dynaconf_hooks.py"
+        )
+        hook_module = py_loader.import_from_filename(
+            obj, hook_file, silent=silent
+        )
+        if not hook_module:
+            # There was no hook on the same path as a python file
+            continue
+        _run_hook_module(
+            hook=hook,
+            hook_module=hook_module,
+            obj=obj,
+            key=key,
+        )
+
+
 def settings_loader(
     obj, settings_module=None, env=None, silent=True, key=None, filename=None
 ):
@@ -94,7 +169,9 @@ def settings_loader(
             # a bare python module name w/o extension
             modules_names.append(item)
 
-    enabled_core_loaders = obj.get("CORE_LOADERS_FOR_DYNACONF")
+    enabled_core_loaders = [
+        item.upper() for item in obj.get("CORE_LOADERS_FOR_DYNACONF") or []
+    ]
 
     # add `.local.` to found_files list to search for local files.
     found_files.extend(
@@ -191,7 +268,7 @@ def write(filename, data, env=None):
     if not loader:
         raise IOError(f"{loader_name} cannot be found.")
 
-    data = DynaBox(data).to_dict()
+    data = DynaBox(data, box_settings={}).to_dict()
     if loader is not py_loader and env and env not in data:
         data = {env: data}
 
