@@ -1,6 +1,22 @@
+from __future__ import annotations
+
 import os
 import warnings
+from collections import defaultdict
 from json import JSONDecoder
+from typing import Any
+from typing import Dict
+from typing import Iterator
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import TYPE_CHECKING
+from typing import Union
+
+
+if TYPE_CHECKING:  # pragma: no cover
+    from dynaconf.utils.boxing import DynaBox
+    from dynaconf.base import LazySettings, Settings
 
 
 BANNER = """
@@ -17,7 +33,9 @@ if os.name == "nt":  # pragma: no cover
     BANNER = "DYNACONF"
 
 
-def object_merge(old, new, unique=False, full_path=None):
+def object_merge(
+    old: Any, new: Any, unique: bool = False, full_path: List[str] = None
+) -> Any:
     """
     Recursively merge two data structures, new is mutated in-place.
 
@@ -26,6 +44,8 @@ def object_merge(old, new, unique=False, full_path=None):
     :param unique: When set to True existing list items are not set.
     :param full_path: Indicates the elements of a tree.
     """
+    if full_path is None:
+        full_path = []
     if old == new or old is None or new is None:
         # Nothing to merge
         return new
@@ -65,7 +85,10 @@ def object_merge(old, new, unique=False, full_path=None):
     return new
 
 
-def recursive_get(obj, names):
+def recursive_get(
+    obj: Union[DynaBox, Dict[str, int], Dict[str, Union[str, int]]],
+    names: Optional[List[str]],
+) -> Any:
     """Given a dot accessible object and a list of names `foo.bar.zaz`
     gets recursivelly all names one by one obj.foo.bar.zaz.
     """
@@ -78,7 +101,9 @@ def recursive_get(obj, names):
     return recursive_get(result, tail)
 
 
-def handle_metavalues(old, new):
+def handle_metavalues(
+    old: Union[DynaBox, Dict[str, int], Dict[str, Union[str, int]]], new: Any
+) -> None:
     """Cleanup of MetaValues on new dict"""
 
     for key in list(new.keys()):
@@ -100,10 +125,9 @@ def handle_metavalues(old, new):
 
         # Data structures containing merge tokens
         if isinstance(new.get(key), (list, tuple)):
-            if (
-                "dynaconf_merge" in new[key]
-                or "dynaconf_merge_unique" in new[key]
-            ):
+            has_merge = "dynaconf_merge" in new[key]
+            has_merge_unique = "dynaconf_merge_unique" in new[key]
+            if has_merge or has_merge_unique:
                 value = list(new[key])
                 unique = False
 
@@ -137,17 +161,33 @@ class DynaconfDict(dict):
     useful to run loaders in to a dict for testing"""
 
     def __init__(self, *args, **kwargs):
+        self._fresh = False
+        self._loaded_envs = []
+        self._loaded_hooks = defaultdict(dict)
+        self._loaded_py_modules = []
         self._loaded_files = []
+        self._deleted = set()
+        self._store = {}
+        self._env_cache = {}
+        self._loaded_by_loaders = {}
+        self._loaders = []
+        self._defaults = {}
+        self.environ = os.environ
+        self.SETTINGS_MODULE = None
+        self.filter_strategy = kwargs.get("filter_strategy", None)
+        self._not_installed_warnings = []
+        self._validate_only = kwargs.pop("validate_only", None)
+        self._validate_exclude = kwargs.pop("validate_exclude", None)
         super(DynaconfDict, self).__init__(*args, **kwargs)
 
-    def set(self, key, value, *args, **kwargs):
+    def set(self, key: str, value: str, *args, **kwargs) -> None:
         self[key] = value
 
     @staticmethod
     def get_environ(key, default=None):  # pragma: no cover
         return os.environ.get(key, default)
 
-    def exists(self, key, **kwargs):
+    def exists(self, key: str, **kwargs) -> bool:
         return self.get(key, missing) is not missing
 
 
@@ -168,7 +208,7 @@ RENAMED_VARS = {
 }
 
 
-def compat_kwargs(kwargs):
+def compat_kwargs(kwargs: Dict[str, Any]) -> None:
     """To keep backwards compat change the kwargs to new names"""
     warn_deprecations(kwargs)
     for old, new in RENAMED_VARS.items():
@@ -186,11 +226,11 @@ class Missing:
     situations where `None` is a valid value.
     """
 
-    def __bool__(self):
+    def __bool__(self) -> bool:
         """Respond to boolean duck-typing."""
         return False
 
-    def __eq__(self, other):
+    def __eq__(self, other: Union[DynaBox, Missing]) -> bool:
         """Equality check for a singleton."""
 
         return isinstance(other, self.__class__)
@@ -198,7 +238,7 @@ class Missing:
     # Ensure compatibility with Python 2.x
     __nonzero__ = __bool__
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """
         Unambiguously identify this string-based representation of Missing,
         used as a singleton.
@@ -209,7 +249,7 @@ class Missing:
 missing = Missing()
 
 
-def deduplicate(list_object):
+def deduplicate(list_object: List[str]) -> List[str]:
     """Rebuild `list_object` removing duplicated and keeping order"""
     new = []
     for item in list_object:
@@ -218,7 +258,7 @@ def deduplicate(list_object):
     return new
 
 
-def warn_deprecations(data):
+def warn_deprecations(data: Any) -> None:
     for old, new in RENAMED_VARS.items():
         if old in data:
             warnings.warn(
@@ -228,7 +268,9 @@ def warn_deprecations(data):
             )
 
 
-def trimmed_split(s, seps=(";", ",")):
+def trimmed_split(
+    s: str, seps: Union[str, Tuple[str, str]] = (";", ",")
+) -> List[str]:
     """Given a string s, split is by one of one of the seps."""
     for sep in seps:
         if sep not in s:
@@ -238,7 +280,7 @@ def trimmed_split(s, seps=(";", ",")):
     return [s]  # raw un-splitted
 
 
-def ensure_a_list(data):
+def ensure_a_list(data: Any) -> Union[List[int], List[str]]:
     """Ensure data is a list or wrap it in a list"""
     if not data:
         return []
@@ -250,7 +292,9 @@ def ensure_a_list(data):
     return [data]
 
 
-def build_env_list(obj, env):
+def build_env_list(
+    obj: Union[Settings, LazySettings], env: Optional[str]
+) -> List[str]:
     """Build env list for loaders to iterate.
 
     Arguments:
@@ -261,30 +305,30 @@ def build_env_list(obj, env):
         [str] -- A list of string names of the envs to load.
     """
     # add the [default] env
-    env_list = [obj.get("DEFAULT_ENV_FOR_DYNACONF")]
+    env_list = [(obj.get("DEFAULT_ENV_FOR_DYNACONF") or "default").lower()]
 
     # compatibility with older versions that still uses [dynaconf] as
     # [default] env
-    global_env = obj.get("ENVVAR_PREFIX_FOR_DYNACONF") or "DYNACONF"
+    global_env = (obj.get("ENVVAR_PREFIX_FOR_DYNACONF") or "dynaconf").lower()
     if global_env not in env_list:
         env_list.append(global_env)
 
     # add the current env
-    if obj.current_env and obj.current_env not in env_list:
-        env_list.append(obj.current_env)
+    current_env = obj.current_env
+    if current_env and current_env.lower() not in env_list:
+        env_list.append(current_env.lower())
 
     # add a manually set env
-    if env and env not in env_list:
-        env_list.append(env)
+    if env and env.lower() not in env_list:
+        env_list.append(env.lower())
 
     # add the [global] env
-    env_list.append("GLOBAL")
+    env_list.append("global")
 
-    # loaders are responsible to change to lower/upper cases
-    return [env.lower() for env in env_list]
+    return env_list
 
 
-def upperfy(key):
+def upperfy(key: str) -> str:
     """Receive a string key and returns its upper version.
 
     Example:
@@ -311,7 +355,7 @@ def upperfy(key):
     return key.upper()
 
 
-def multi_replace(text, patterns):
+def multi_replace(text: str, patterns: Dict[str, str]) -> str:
     """Replaces multiple pairs in a string
 
     Arguments:
@@ -326,7 +370,9 @@ def multi_replace(text, patterns):
     return text
 
 
-def extract_json_objects(text, decoder=JSONDecoder()):
+def extract_json_objects(
+    text: str, decoder: JSONDecoder = JSONDecoder()
+) -> Iterator[Dict[str, Union[int, Dict[Any, Any]]]]:
     """Find JSON objects in text, and yield the decoded JSON data
 
     Does not attempt to look for JSON arrays, text, or other JSON types outside
@@ -346,7 +392,9 @@ def extract_json_objects(text, decoder=JSONDecoder()):
             pos = match + 1
 
 
-def recursively_evaluate_lazy_format(value, settings):
+def recursively_evaluate_lazy_format(
+    value: Any, settings: Union[Settings, LazySettings]
+) -> Any:
     """Given a value as a data structure, traverse all its members
     to find Lazy values and evaluate it.
 
