@@ -5,9 +5,11 @@ import warnings
 from functools import wraps
 
 from dynaconf.utils import extract_json_objects
+from dynaconf.utils import isnamedtupleinstance
 from dynaconf.utils import multi_replace
 from dynaconf.utils import recursively_evaluate_lazy_format
 from dynaconf.utils.boxing import DynaBox
+from dynaconf.utils.functional import empty
 from dynaconf.vendor import toml
 
 try:
@@ -79,6 +81,9 @@ class Merge(MetaValue):
     _dynaconf_merge = True
 
     def __init__(self, value, box_settings, unique=False):
+        if unique:
+            self._dynaconf_merge_unique = True
+
         self.box_settings = box_settings
 
         self.value = parse_conf_data(
@@ -161,19 +166,26 @@ class Lazy:
 
     _dynaconf_lazy_format = True
 
-    def __init__(self, value, formatter=Formatters.python_formatter):
+    def __init__(
+        self, value=empty, formatter=Formatters.python_formatter, casting=None
+    ):
         self.value = value
         self.formatter = formatter
+        self.casting = casting
 
     @property
     def context(self):
         """Builds a context for formatting."""
         return {"env": os.environ, "this": self.settings}
 
-    def __call__(self, settings):
+    def __call__(self, settings, validator_object=None):
         """LazyValue triggers format lazily."""
         self.settings = settings
-        return self.formatter(self.value, **self.context)
+        self.context["_validator_object"] = validator_object
+        result = self.formatter(self.value, **self.context)
+        if self.casting is not None:
+            result = self.casting(result)
+        return result
 
     def __str__(self):
         """Gives string representation for the object."""
@@ -187,6 +199,11 @@ class Lazy:
         """Encodes this object values to be serializable to json"""
         return f"@{self.formatter} {self.value}"
 
+    def set_casting(self, casting):
+        """Set the casting and return the instance."""
+        self.casting = casting
+        return self
+
 
 def try_to_encode(value, callback=str):
     """Tries to encode a value by verifying existence of `_dynaconf_encode`"""
@@ -209,10 +226,25 @@ def evaluate_lazy_format(f):
 
 
 converters = {
-    "@int": int,
-    "@float": float,
-    "@bool": lambda value: str(value).lower() in true_values,
-    "@json": json.loads,
+    "@str": lambda value: value.set_casting(str)
+    if isinstance(value, Lazy)
+    else str(value),
+    "@int": lambda value: value.set_casting(int)
+    if isinstance(value, Lazy)
+    else int(value),
+    "@float": lambda value: value.set_casting(float)
+    if isinstance(value, Lazy)
+    else float(value),
+    "@bool": lambda value: value.set_casting(
+        lambda x: str(x).lower() in true_values
+    )
+    if isinstance(value, Lazy)
+    else str(value).lower() in true_values,
+    "@json": lambda value: value.set_casting(
+        lambda x: json.loads(x.replace("'", '"'))
+    )
+    if isinstance(value, Lazy)
+    else json.loads(value),
     "@format": lambda value: Lazy(value),
     "@jinja": lambda value: Lazy(value, formatter=Formatters.jinja_formatter),
     # Meta Values to trigger pre assignment actions
@@ -228,6 +260,7 @@ converters = {
     "@comment": lambda value: None,
     "@null": lambda value: None,
     "@none": lambda value: None,
+    "@empty": lambda value: empty,
 }
 
 
@@ -272,10 +305,22 @@ def _parse_conf_data(data, tomlfy=False, box_settings=None):
         and isinstance(data, str)
         and data.startswith(tuple(converters.keys()))
     ):
-        parts = data.partition(" ")
-        converter_key = parts[0]
-        value = parts[-1]
-        value = get_converter(converter_key, value, box_settings)
+        # Check combination token is used
+        comb_token = re.match(
+            r"^@(str|int|float|bool|json) @(jinja|format)", data
+        )
+        if comb_token:
+            tokens = comb_token.group(0)
+            converter_key_list = tokens.split(" ")
+            value = data.replace(tokens, "").strip()
+        else:
+            parts = data.partition(" ")
+            converter_key_list = [parts[0]]
+            value = parts[-1]
+
+        # Parse the converters iteratively
+        for converter_key in converter_key_list[::-1]:
+            value = get_converter(converter_key, value, box_settings)
     else:
         value = parse_with_toml(data) if tomlfy else data
 
@@ -287,10 +332,15 @@ def _parse_conf_data(data, tomlfy=False, box_settings=None):
 
 def parse_conf_data(data, tomlfy=False, box_settings=None):
 
+    # fix for https://github.com/rochacbruno/dynaconf/issues/595
+    if isnamedtupleinstance(data):
+        return data
+
     # not enforced to not break backwards compatibility with custom loaders
     box_settings = box_settings or {}
 
     if isinstance(data, (tuple, list)):
+
         # recursively parse each sequence item
         return [
             parse_conf_data(item, tomlfy=tomlfy, box_settings=box_settings)
@@ -306,6 +356,13 @@ def parse_conf_data(data, tomlfy=False, box_settings=None):
             )
         return _parsed
 
+    if (
+        isinstance(data, str)
+        and data.startswith(("+", "-"))
+        and data[1:].isdigit()
+    ):
+        return data
+
     # return parsed string value
     return _parse_conf_data(data, tomlfy=tomlfy, box_settings=box_settings)
 
